@@ -99,10 +99,7 @@ method_changed (CEPageIP4 *self)
         } else {
                 addr_enabled = g_str_equal (method, "manual");
                 routes_enabled = !g_str_equal (method, "local");
-                if (g_str_equal (method, "local"))
-                        dns_enabled = FALSE;
-                else
-                        dns_enabled = !gtk_switch_get_active (self->auto_dns_switch);
+                dns_enabled = !g_str_equal (method, "local");
         }
 
         gtk_widget_set_visible (GTK_WIDGET (self->address_box), addr_enabled);
@@ -319,7 +316,7 @@ add_dns_section (CEPageIP4 *self)
         gint i;
 
         gtk_switch_set_active (self->auto_dns_switch, !nm_setting_ip_config_get_ignore_auto_dns (self->setting));
-        g_signal_connect_object (self->auto_dns_switch, "notify::active", G_CALLBACK (method_changed), self, G_CONNECT_SWAPPED);
+        g_signal_connect_object (self->auto_dns_switch, "notify::active", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
 
         string = g_string_new ("");
 
@@ -608,9 +605,8 @@ ui_to_setting (CEPageIP4 *self)
 
         dns_servers = g_ptr_array_new_with_free_func (g_free);
         dns_text = g_strstrip (g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->dns_entry))));
-        if ((g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO) ||
-            g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL)) &&
-            !gtk_switch_get_active (self->auto_dns_switch))
+        if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO) ||
+            g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
                 dns_addresses = g_strsplit_set (dns_text, ", ", -1);
         else
                 dns_addresses = NULL;
